/*****************************************************************************/
/*                                                                           */
/*   Data Acquisition System Control Program        Version 1.0.0            */
/*                     for Linux-2.0.31 (slockware)                          */
/*                                                                           */
/*         Produced on 3 Dec 1997 by  Naohito Iwasa                          */
/*                                                                           */
/*****************************************************************************/
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include "camac.h"
#include "blocker.h"

#define WAIT 10000L

int shmid;
struct SHM_CC7700 *shmp;

void tolowers(char str[],int n)
{
  int i;
  for(i=0;i<n;i++){
    if(str[i]>='A'&&str[i]<='Z')str[i]+='a'-'A';
    if(str[i]==0x0a)str[i]=0;
  }
}
char *time2asc(time_t time)
{
  static char str[40];
  struct tm *t;
  if(time==0)strcpy(str,"--:--:--");
  else{
    t=localtime(&time);
    sprintf(str,"%02d:%02d:%02d\0",t->tm_hour,t->tm_min,t->tm_sec);
  }
  return(str);
}
char *date2asc(time_t time)
{
  static char str[40];
  struct tm *t;
  char *month[12]={
    "JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};
  if(time==0)strcpy(str,"--------");
  else{
    t=localtime(&time);
    sprintf(str,"%02d-%s-%02d\0",
            t->tm_mday,month[t->tm_mon],t->tm_year%100);
  }
  return(str);
}

int main(int argc, char *argv[])
{
  int fd, i, j;
  long arg;
  DCCycle dcc;
  char line[80];
  time_t current;
  unsigned int k;

  if ((shmid = shmget(SHMKEY, sizeof(struct SHM_CC7700), 0600)) != -1){
    shmp = (struct SHM_CC7700 *)shmat(shmid, 0, 0);
  }
  
  if((fd = open("/dev/PCIcc7700", O_RDWR))<0){
    puts("****** device /dev/PCIcc7700 cannot be opened ******");
    exit(1);
  }
  ioctl(fd, DC_INITIAL, &i);
  /* printf("%d",i);*/
  
L0:
  if(shmp->scaler_max<0){
    shmp->scaler_max=0;
    for(i=0;i<10;i++){
      printf("Scaler%02d: station number (end=0)?",i+1);
      fgets(line,80,stdin);
      if(atoi(line)<=0||atoi(line)>23)break;
      shmp->scalers[i]=atoi(line);
      shmp->scaler_max++;
    }
  }
  while(1){
    printf("COM %% ");
    if(fgets(line,80,stdin)==NULL)break;
L1:
    tolowers(line,80);
    if(strncmp(line,"start",3)==0){
      if(shmp->status!=0)
	puts("acquisition is already running"); 
      else{
	shmp->com_message[0]=0;
	time(&shmp->com_time);
	shmp->com_flag=1;
	ioctl(fd, DC_INTON, 0);
	while(shmp->com_flag!=0)usleep(1000);
      }
    }
    else if(strncmp(line,"stop",3)==0){
      if(shmp->status==0)
	puts("no acquisition ran"); 
      else{
	shmp->com_message[0]=0;
	time(&shmp->com_time);
	shmp->com_flag=1;
	ioctl(fd, DC_INTOFF, 0);
	while(shmp->com_flag!=0){
	  sleep(1);
	  ioctl(fd, DC_INTOFF, 0);
	}
      }
    }
    else if(strncmp(line,"open",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd <= 0){
	shmp->com_message[0]=0;
	printf("open RUN-%04d file=%s.%03d\n",shmp->runno+1,
	       shmp->com_filename,shmp->runno+1);
	printf("Comment for the begining of the RUN\n");
	fgets(shmp->com_comments,256,stdin);
	time(&shmp->com_time);
	shmp->com_flag=1;
	shmp->runno++;
	ioctl(fd, DC_INTOPN, 0);
	while(shmp->com_flag!=0)usleep(1000);
	if(shmp->com_message[0]!=0)puts(shmp->com_message);
      }
    }
    else if(strncmp(line,"close",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd > 0){
	shmp->com_message[0]=0;
	printf("close RUN-%04d file=%s\n",shmp->runno,shmp->raw_filename);
	printf("Comment for the end of RUN\n");
	fgets(shmp->com_comments,256,stdin);
	time(&shmp->com_time);
	shmp->com_flag=1;
	ioctl(fd, DC_INTEOF, 0);
	while(shmp->com_flag!=0)usleep(1000);
	if(shmp->com_message[0]!=0)puts(shmp->com_message);
      }
    }
    else if(strncmp(line,"clear",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd > 0)
	puts("output file is opened !! close first!!");
      else {
	shmp->com_message[0]=0;
	shmp->com_flag=1;
	ioctl(fd, DC_INTCLR, 0);
	for(i=0;i<shmp->scaler_max;i++)
	  for(j=0;j<12;j++)
	    shmp->scaler_data[i][j]=0;
	while(shmp->com_flag!=0)usleep(1000);
      }
      goto L0;
    }
    else if(strncmp(line,"run",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd > 0)
	puts("output file is opened !! close first!!");
      else {
	shmp->com_message[0]=0;
	printf("RUN No. = ");
	fgets(line,80,stdin);
	if(atoi(line)>0)shmp->runno=atoi(line)-1;
      }
    }
    else if(strncmp(line,"set",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd > 0)
	puts("output file is opened !! close first!!");
      else {
	shmp->com_message[0]=0;
	printf("project filename ? ");
	fgets(shmp->com_filename,256,stdin);
	for(i=0;i<256;i++){
	  if(shmp->com_filename[i]<0x20)shmp->com_filename[i]=0;
	}
	shmp->runno=0;
	strcpy(line,"clear");
	goto L1;
      }
    }
    else if(strncmp(line,"reset",3)==0){
      if(shmp->status!=0)
	puts("acquisition still running !! Stop first!!"); 
      else if(shmp->raw_fd > 0)
	puts("output file is opened !! close first!!");
      else {
	shmp->com_message[0]=0;
	shmp->scaler_max=0;
	for(i=0;i<10;i++){
	  printf("Scaler%02d: station number (end=0)?",i+1);
	  fgets(line,80,stdin);
	  if(atoi(line)<=0||atoi(line)>23)break;
	  shmp->scalers[i]=atoi(line);
	  shmp->scaler_max++;
	}
      }
    }
    else if(strncmp(line,"sca",3)==0){
      shmp->com_message[0]=0;
      for(i=0;i<shmp->scaler_max;i++)printf("**SCA%02d**",i);
      printf("\n");
      for(i=0;i<shmp->scaler_max;i++)printf("  N=%02d   ",shmp->scalers[i]);
      printf("\n");
      
      for(j=0;j<12;j++){
	for(i=0;i<shmp->scaler_max;i++){
	  dcc.n=shmp->scalers[i];
	  dcc.f=0;
	  dcc.a=j;
	  ioctl(fd, DC_CYCLE, &dcc);
	  dcc.data+=shmp->scaler_data[i][j];
	  printf(" %8d",dcc.data); 
	}
	printf("\n");
      }
    }
 
    else if(line[0]==0){
      shmp->com_message[0]=0;
      time(&current);
      if(shmp->status==0){
	printf(" Run-%04d   Start = %s  ",
	       shmp->runno, time2asc(shmp->start_time));
	printf("Stop = %s  ",time2asc(shmp->stop_time));
	printf("Current= %s  %s\n",time2asc(current),date2asc(current));
      }
      else{
	printf(" Run-%04d   Start = %s  ***ACQUIRING***  ",
	       shmp->runno, time2asc(shmp->start_time));
	printf("Current= %s  %s\n",time2asc(current),date2asc(current));
      }
      if(shmp->raw_fd>0)
	printf(" FILENAME = %s\n",shmp->raw_filename);
      ioctl(fd, DC_INTCNT, &k);
      printf("ACQUIRE EVENTS=%10d\n",k);
      printf("BUFFER  EVENTS=%10d\n",shmp->event_no);
      printf("BUFFER  BLOCKS=%10d\n",shmp->block_no);
    }
    else{
      puts("************ interrupt test **************");
      puts(" start: interrupt on ");
      puts(" stop : interrupt off ");
      puts(" open : file open ");
      puts(" close: file close ");
      puts(" clear: clear acquisition table and scalers");
      puts(" C/R  : status ");
      puts(" sca  : show scaler value ");
      puts(" reset: reset scaler station number");
      puts(" set  : set filename");
      puts(" run  : set run number number");
    }
  }
  return 0;
}	


